#ifndef _VWMTERM_H_
#define _VWMTERM_H_

#include <glib.h>
#include <gmodule.h>
#include <termios.h>

#include <sys/ioctl.h>

#include <vwm.h>
#include <viper.h>
#include <pseudo.h>

#ifdef _VIPER_WIDE
#include <ncursesw/curses.h>
#else
#include <curses.h>
#endif

#include "vscreen.h"

/* define PTY_READ_MAX for the most liberal value.  fallback to the (POSIX).
   the most portable value. in the end, PTY_READ_MAX _should_ reflect the
   maximum number of bytes that the system is claiming to write to pipes
   atomically.  it's not necessary to read in atomic chunks but it seems wise
   to harmonize the pty read buffer size with this value.  */
#ifdef   PIPE_BUF
#define  PTY_READ_MAX         PIPE_BUF
#endif
#ifndef  PTY_READ_MAX
#ifdef   _POSIX_PIPE_BUF
#define  PTY_READ_MAX         _POSIX_PIPE_BUF   
#endif
#endif
#ifndef  PTY_READ_MAX
#define  PTY_READ_MAX         512
#endif

#define	PTY_FG					(1<<1)   /* tty is in the foreground   */

#define 	SCROLLBACK	0
#define	WHITEBOARD	1

typedef struct
{
	vscreen_t		*vscreens[2];
	WINDOW			*window;
	guint				scr_idx;
	pid_t				child_pid;
	gint				master_fd;
	gint				slave_fd;
	gchar				term_name[64];
	struct winsize	win_sz;
	gint				new_state;			/* these are PID & PTY state flags.		*/
	gint				old_state;			/* screen flags are located elsewhere.	*/
   gchar          *carryover;
   gint           carry_sz;
}
vwmterm_t;

#define	VWMTERM_SCREEN(x)	(x->vscreens[x->scr_idx])
#define 	VWMTERM_WINDOW(x)	((x->vscreens[x->scr_idx])->window)


G_MODULE_EXPORT gchar* g_module_check_init(GModule *module);

WINDOW*	vwmterm_init(gpointer anything);
void 		vwmterm_activate_vscreen(vwmterm_t *vwmterm,guint scr_id);

#endif
