#include <unistd.h>
#include <termios.h>

#include <viper.h>

#include "vscreen.h"

static void whiteboard_activate(vscreen_t *whiteboard);
static void whiteboard_addch(vscreen_t *whiteboard,chtype ch);
static void	whiteboard_move(vscreen_t *whiteboard,gint y,gint x);

vscreen_t* whiteboard_vscreen_from(WINDOW *source,guint pty_fd)
{
	vscreen_t		*whiteboard;
	WINDOW			*window;
	struct termios	*term_desc;

	if(source==NULL) return NULL;
	window=dupwin(source);

	whiteboard=(vscreen_t*)g_malloc0(sizeof(vscreen_t));
	VSCREEN(whiteboard)->pty_fd=pty_fd;

	scrollok(window,TRUE);
	idlok(whiteboard->window,FALSE);
	wattrset(window,VIPER_COLORS(COLOR_WHITE,COLOR_BLACK));
	wbkgdset(window,VIPER_COLORS(COLOR_WHITE,COLOR_BLACK));
	werase(window);

	term_desc=&whiteboard->term_desc;
	tcgetattr(pty_fd,term_desc);

 	term_desc->c_lflag |= ISIG | IEXTEN;
	term_desc->c_lflag &= ~ICANON & ~ECHO;
	term_desc->c_iflag |= BRKINT | IGNPAR | IXON | ICRNL;
	term_desc->c_iflag &= ~IXANY & ~INLCR & ~IGNCR;

	whiteboard->window=window;
	whiteboard->state |= VSCREEN_SCROLLOK;
	whiteboard->fg=COLOR_WHITE;
	whiteboard->bg=COLOR_BLACK;
	whiteboard->activate=whiteboard_activate;
	whiteboard->write_char=whiteboard_addch;
	whiteboard->move_cursor=whiteboard_move;

	return whiteboard;
}

static void whiteboard_activate(vscreen_t *whiteboard)
{
	tcsetattr(whiteboard->pty_fd,TCSAFLUSH,&whiteboard->term_desc);

	wattron(whiteboard->window,A_NORMAL);
	werase(whiteboard->window);

	scrollok(whiteboard->window,TRUE);
	whiteboard->state |= VSCREEN_SCROLLOK;
}

static void whiteboard_addch(vscreen_t *whiteboard,chtype ch)
{
	gint     cur_x,cur_y;

   getyx(whiteboard->window,cur_y,cur_x);


	if(ch=='\r')
   {
      whiteboard->move_cursor(whiteboard,cur_y,0);
      return;
   }

	if(ch=='\n')
   {
      if(is_curs_at_bottom(whiteboard->window)!=-1)
      {
         scroll(whiteboard->window);
         wmove(whiteboard->window,cur_y,0);
         return;
      }

      wmove(whiteboard->window,cur_y+1,0);
      return;
   }

	waddch(whiteboard->window,ch);

	return;
}

static void whiteboard_move(vscreen_t *whiteboard,gint y,gint x)
{
	wmove(whiteboard->window,y,x);

	return;
}


