/*************************************************************************
 * All portions of code are copyright by their respective author/s.
 * Copyright (C) 2007      Bryan Christ <bryan.christ@hp.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------*/

#include <string.h>
#include <stdio.h>

#include <viper.h>

#include "vwmterm.h"
#include "vscreen.h"
#include "xlate.h"

gint vwmterm_xlate_esc(vwmterm_t *vwmterm,gchar *buffer)
{
	static gchar	codes[]={'l','d','h','c','m','H','K','J','L','C','P','X'};
	vscreen_t		*vscreen;
	WINDOW			*window;
	gint		      array_size=0;
	gint		      size=0;
	gint		      idx;
	gint		      max_x,max_y;
	gint		      cur_x,cur_y;
	gint		      x=0,y=0;
	gint		      value1;
	gchar		      *token;
	gchar		      *control=NULL;

	vscreen=VWMTERM_SCREEN(vwmterm);
	window=VWMTERM_WINDOW(vwmterm);
	getmaxyx(window,max_y,max_x);
	getyx(window,cur_y,cur_x);

	/*	if its not an escape code, bail	*/
	if(buffer[0]!=27) return 0;

	/* incomplete buffer transmission	*/
	if(buffer[0]==27 && buffer[1]=='\0') return -1;

   /* handle a terminal reset (RIS) */
   if(buffer[0]==27 && buffer[1]=='c')
   {
      /* flush the master and slave pty fds when the terminal is reset.
         not sure if this is necessary, but it doesn't seem to hurt.  */
      tcdrain(vwmterm->slave_fd);
      tcdrain(vwmterm->master_fd);
      return 2;
   }

	/* it's an escape code, but not one defined by vwmterm	*/
	if(buffer[0]==27 && buffer[1]!='[') return 0;

	buffer+=2;

	array_size=sizeof(codes)/sizeof(codes[0]);
	for(idx=0;idx<array_size;idx++)
	{
		token=strchr(buffer,codes[idx]);
		if(token!=NULL)
		{
			if(control==NULL) control=token;
			if(token<control) control=token;
		}
	}
   /* legitimate escape code suffixes are defined in the terminfo file
      for vwmterm.  if we do not find one of these suffixes, then our async
      buffer was cut during the middle of an escape transmission and so we need
      to carryover the remainder. */
	if(control==NULL) return -1;
	size=((control-buffer)/sizeof(gchar))+3;

	if(*control=='h')
	{
      /* turn on auto margins capability (rmam)
			which is = scrollok(WINDOW *,TRUE) */
		if(memcmp(buffer,"?7h",3)==0) vscreen_scrollok(vscreen,TRUE);

      /* make cursor visible (cnorm)   */
      if(memcmp(buffer,"?25h",4)==0) vscreen_show_cursor(vscreen,TRUE);

		/* start cursor contorl mode (ncurses initscr) (smcup)   */
		if(memcmp(buffer,"?1049h",6)==0)
			vwmterm_activate_vscreen(vwmterm,WHITEBOARD);

		return size;
	}

	if(*control=='d')
	{
		value1=atoi(buffer)-1;
		vscreen_move_cursor(vscreen,value1,cur_x);

		return size;
	}

	if(*control=='l')
	{
      /* turn off auto margins capability (smam) */
 		if(memcmp(buffer,"?7l",3)==0) vscreen_scrollok(vscreen,FALSE);

      /* make cursor invisible (civis) */
      if(memcmp(buffer,"?25l",4)==0) vscreen_show_cursor(vscreen,FALSE);

		/* end cursor contorl mode (ncurses endwin) (rmcup)   */
		if(memcmp(buffer,"?1049l",6)==0)
			vwmterm_activate_vscreen(vwmterm,SCROLLBACK);

		return size;
	}

	if(*control=='m')
	{
		if(buffer==control) vwmterm_xlate_sgr(vwmterm,0);
		while(buffer!=control)
		{
			value1=atoi(buffer);
			vwmterm_xlate_sgr(vwmterm,value1);
			token=strchr(buffer,';');
			if(token==NULL) break;
			if(token<control) buffer=token;
			if(token>control) break;
			buffer++;
		}
		return size;
	}

	if(*control=='H')
	{
		if(buffer==control)
		{
			vscreen_move_cursor(vscreen,0,0);
			return size;
		}

		y=atoi(buffer)-1;
		token=(strchr(buffer,';'));
		if(token==NULL) return size;
		buffer=token;
		buffer++;
		x=atoi(buffer)-1;
		if(x<0) x=0;
		if(y<0) y=0;

		vscreen_move_cursor(vscreen,y,x);
		return size;
	}

	if(*control=='J')
	{
      if(memcmp(buffer,"2J",2)==0) vscreen_move_cursor(vscreen,0,0);
      wclrtobot(window);

//		flash();
		return size;
	}

	if(*control=='K')
	{
		if(buffer[0]=='K')
		{
         wclrtoeol(window);
         wchgat(window,-1,A_NORMAL,
				viper_color_pair(vscreen->fg,vscreen->bg),NULL);
		}

//		flash();
		return size;
	}

	if(*control=='P')
	{
		flash();
		return size;
	}

	if(*control=='X')
	{
		flash();
		return size;
	}

	if(*control=='L')
	{
		/* if(buffer[0]=='L') wscrl(window,-1); */
		if(buffer[0]=='L') winsertln(window);

		return size;
	}

   if(*control=='C')
   {
      /* this should never happend  */
//    if(buffer==control) return size;
		if(buffer==control) x=1;
		else
		{
      	x=atoi(buffer);
	   	if(x<0) x=0;
			vscreen_move_cursor(vscreen,cur_y,cur_x+x);
		}
      return size;
   }

	return -1;
}

gint	vwmterm_xlate_specials(vwmterm_t *vwmterm,gchar *buffer)
{
	vscreen_t	*vscreen;
	WINDOW		*window;
	gint        cur_x,cur_y;
	gint	      max_x,max_y;
	guint	      code;

	code=(guint)buffer[0];
	if(code>31 && code <127) return 0;

	vscreen=VWMTERM_SCREEN(vwmterm);
	window=VWMTERM_WINDOW(vwmterm);
	getmaxyx(window,max_y,max_x);
	getyx(window,cur_y,cur_x);

	switch(code)
	{
		case 0: return 0;
		case 3: return 1;
		case 7:
		{
			beep();
			return 1;
		}
		case 8:
		{
			vscreen_addch(vscreen,8);
			return 1;
		}
		case 10: /* NL or LF (new line / line feed)  */
		{
			vscreen_addch(vscreen,'\n');
			return 1;
		}
		case 13: /* CR (carriage return) */
		{
			vscreen_addch(vscreen,'\r');
			return 1;
		}
		case 14:
		{
			vscreen_move_cursor(vscreen,cur_y+1,cur_x);
			return 1;
		}
		case 15: return 1;
		case 16:
		{
			flash();
			vscreen_move_cursor(vscreen,cur_y-1,cur_x);
			return 1;
		}
      /* don't process 27 because it is the escape code for more complex
         control sequences.   */
		case 27: return 0;
      /* ignore odd escape codes that aren't currently handled.   */
		default: return 1;
	}

	return 1;
}

void vwmterm_xlate_sgr(vwmterm_t *vwmterm,gint code)
{
	static gint		colors[]={COLOR_BLACK,COLOR_RED,COLOR_GREEN,COLOR_YELLOW,
							COLOR_BLUE,COLOR_MAGENTA,COLOR_CYAN,COLOR_WHITE};
	vscreen_t		*vscreen;
	WINDOW			*window;

	vscreen=VWMTERM_SCREEN(vwmterm);
	window=VWMTERM_WINDOW(vwmterm);

	if(code==0)
	{
		wattrset(window,A_NORMAL);
		vscreen->fg=COLOR_WHITE;
		vscreen->bg=COLOR_BLACK;
		wattron(window,VIPER_COLORS(COLOR_WHITE,COLOR_BLACK));
		return;
	}

	if(code==1) wattron(window,A_BOLD);       /* bold  */
	if(code==2) wattron(window,A_DIM);        /* dim   */
	if(code==3) wattron(window,A_STANDOUT);   /* standout */
	if(code==4) wattron(window,A_UNDERLINE);  /* underline   */
	if(code==5) wattron(window,A_BLINK);      /* blink */
	if(code==6) wattron(window,A_BLINK);      /* blink */
	if(code==7) wattron(window,A_REVERSE);    /* reverse  */
	if(code==8) wattron(window,A_INVIS);      /* invisible   */
	/*	codes 10 and 11 are reserved in the ansi spec so they get used
		for entrance and exit escapes for ALT CHARSET.  this is defined in
		the terminfo for vwmterm.  everybody else is doing it so why
		can't i? :)	*/
	if(code==10) vscreen->state &= (~VSCREEN_ACS);
	if(code==11) vscreen->state |= VSCREEN_ACS;
	if(code==23) wattroff(window,A_STANDOUT);
	if(code==24) wattroff(window,A_UNDERLINE);
	if(code==25) wattroff(window,A_BLINK);
	if(code==27) wattroff(window,A_REVERSE);
	if(code>=30 && code<=37) vscreen->fg=colors[code-30];
	if(code==39) vscreen->fg=COLOR_WHITE;	            /* hard-coded default	*/ 
	if(code>=40 && code<=47) vscreen->bg=colors[code-40];
	if(code==49) vscreen->bg=COLOR_BLACK;	            /* hard-coded default   */

	if(code>=30 && code<=49)
		wattron(window,VIPER_COLORS(vscreen->fg,vscreen->bg));

	return;
}

/*	some programs try to second-guess ncurses.  this is really unfortunate
	because such programs can generate escape codes that are unhandled.  the
	effects of these codes on the parser are generally not good.  this
	function attempts to discard these bad values.	*/
gint vwmterm_xlate_discards(gchar *buffer)
{
	static gchar	*discards[]={
				   "\x1B[?1000h","\x1B)0","\x1B[?1000l","\x1B[?9h","\x1B[?9l",
				   "\x1B\x38","\x1B\x37"};
   static guint   array_sz=sizeof(discards)/sizeof(discards[0]);
	gint	         size=0;
	guint	         i=0;


   for(i=0;i<array_sz;i++)
   {
      size=strlen(discards[i]);
      if(memcmp(discards[i],buffer,size)==0) break;
   }

   if(i==array_sz) return 0;
	return size;
}
